import {
  csv,
  select,
  symbolCircle,
  symbolSquare,
  symbolStar,
} from 'd3';
import { scatterPlot } from './scatterPlot';

const csvUrl =
  'https://gist.githubusercontent.com/sagam929/4900c59d0a817c635282039a08cec177/raw/f9833cb9cd956241f0b6dd0e0a8bb00fa77e1293/NoStuGoingToDifferentCountry.csv';

// Define symbol mapping
const symbolMapping = {
  Bacherlors: symbolCircle,
  Masters: symbolSquare,
  PHD: symbolCircle,
  Jobs: symbolSquare,
};

// Define symbol mapping
const types = {
  0: 'Bacherlors',
  1: 'Masters',
  2: 'PHD',
  3: 'Jobs',
};

const svg = select('body')
  .append('svg')
  .attr('width', window.innerWidth)
  .attr('height', window.innerHeight);

const parseRow = (d) => {
  d.Year = +d.Year;
  Object.keys(d).forEach((k) => {
    if (k !== 'Year' && k !== 'Entity' && k !== 'Code')
      d[k] = +d[k];
  });
  return d;
};

const main = async () => {
  const data = await csv(csvUrl, parseRow);
  const cancerTypes = Object.keys(symbolMapping);
  let index = 0; // To cycle through cancer types
  let counter = 0; // To cycle through cancer types
  const updatePlot = () => {
    const cancerType =
      cancerTypes[index % cancerTypes.length];
    const updatedData = data.map((d) => ({
      Year: d.Year,
      Deaths: d[cancerType],
    }));
    scatterPlot()
      .width(window.innerWidth)
      .height(window.innerHeight)
      .data(updatedData)
      .xValue((d) => d.Year)
      .yValue((d) => d.Deaths)
      .margin({
        top: 100,
        right: 20,
        bottom: 100,
        left: 150,
      })
      .symbolType(() => symbolMapping[cancerType])(
      // Use the symbol mapping
      svg,
    );
    index++;
    if (index < cancerTypes.length) {
      svg.selectAll('.cancer-title').remove();
      setTimeout(updatePlot, 2000); // Update every 3 seconds
      svg
        .append('text')
        .attr('class', 'cancer-title')
        .attr('x', 350)
        .attr('y', 90)
        .text(types[index - 1] + ' Student ')
        .style('font-size', '40px')
        .attr('fill', 'red')
        .attr('alignment-baseline', 'middle');
      counter++;
    } else {
      svg.selectAll('.cancer-title').remove();
      svg
        .append('text')
        .attr('class', 'cancer-title')
        .attr('x', 350)
        .attr('y', 90)
        .text(types[index - 1] + ' Students ')
        .style('font-size', '40px')
        .attr('fill', 'red')
        .attr('alignment-baseline', 'middle');
      index = 0;
      setTimeout(updatePlot, 2000);
    }
  };
  updatePlot();
};

main();
