import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  select,
  symbol,
  symbolCircle,
  symbolSquare,
  symbolStar,
  scaleOrdinal,
  schemeCategory10,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin = { top: 10, right: 10, bottom: 50, left: 50 }; // Adjusted margins for axis labels
  let symbolType = symbolCircle; // Default symbol type
  let color = scaleOrdinal(schemeCategory10);

  const chart = (selection) => {
    selection.each(function () {
      // Create scales
      const xScale = scaleLinear()
        .domain(extent(data, xValue))
        .range([0, width - margin.left - margin.right]);
      const yScale = scaleLinear()
        .domain(extent(data, yValue))
        .range([height - margin.top - margin.bottom, 0]);

      // Create the root container
      const svg = select(this)
        .selectAll('svg')
        .data([null]);
      const svgEnter = svg.enter().append('svg');
      svgEnter
        .merge(svg)
        .attr('width', width)
        .attr('height', height);

      // Append a 'g' element for chart
      const gEnter = svgEnter.append('g');
      const g = svg.select('g').merge(gEnter);
      g.attr(
        'transform',
        `translate(${margin.left},${margin.top})`,
      );

      // X-axis and the x-axis label
      const xAxis = axisBottom(xScale).ticks(12);
      const xAxisGEnter = gEnter
        .append('g')
        .attr('class', 'x-axis')
        .attr(
          'transform',
          `translate(0,${height - margin.top - margin.bottom})`,
        );
      xAxisGEnter
        .call(xAxis)
        .selectAll('text')
        .attr('transform', 'rotate(-90)')
        .attr('dy', '0.5em')
        .attr('dx', '-1em') // Adjust this value as needed
        .style('text-anchor', 'end')
        .style('font-size', '20px');

      // Append x-axis label
      xAxisGEnter
        .append('text')
        .attr('class', 'axis-label')
        .attr('x', width - margin.left - margin.right) // Position the label to the right
        .attr('y', 40) // Adjust the vertical position as needed
        .style('text-anchor', 'end')
        .style('font-size', '20px')
        .text('Years'); // Set x-axis label text to "Years"

      // Y-axis
      const yAxis = axisLeft(yScale).ticks(12);
      const yAxisGEnter = gEnter
        .append('g')
        .attr('class', 'y-axis');
      yAxisGEnter
        .call(yAxis)
        .append('text')
        .attr('class', 'axis-label')
        .attr('transform', 'rotate(-90)')
        .attr('y', -90) // Adjust this to move the label to the left of the y-axis line
        .attr(
          'x',
          -(height - margin.top - margin.bottom) / 2,
        ) // Center the label
        .attr('text-anchor', 'middle')
        .attr('fill', 'red')
        .text('No of Students Going Abroad'); // Text for the y-axis label

      // Call the y-axis on update
      g.select('.y-axis').transition().call(yAxis);

      // Draw symbols
      const symbols = g.selectAll('.symbol').data(data);

      symbols
        .enter()
        .append('path')
        .attr('class', 'symbol')
        .merge(symbols)
        .attr('d', symbol().size(100).type(symbolType)) // Increase the size to 100 (adjust as needed)
        .attr(
          'transform',
          (d) =>
            `translate(${xScale(xValue(d))},${yScale(yValue(d))})`,
        )
        .attr('fill', (d) => {
          // Assuming you have a 'type' property in your data determining the color
          const type = d.type;
          // Use the color scale to determine the fill color based on the 'type' property
          return color(type);
        });

      symbols.exit().remove();
    });
  };

  chart.color = (_) =>
    arguments.length ? ((color = _), chart) : color;
  chart.width = (_) =>
    arguments.length ? ((width = +_), chart) : width;
  chart.height = (_) =>
    arguments.length ? ((height = +_), chart) : height;
  chart.data = (_) =>
    arguments.length ? ((data = _), chart) : data;
  chart.xValue = (_) =>
    arguments.length ? ((xValue = _), chart) : xValue;
  chart.yValue = (_) =>
    arguments.length ? ((yValue = _), chart) : yValue;
  chart.margin = (_) =>
    arguments.length ? ((margin = _), chart) : margin;
  chart.radius = (_) =>
    arguments.length ? ((radius = +_), chart) : radius;
  chart.symbolType = (_) =>
    arguments.length
      ? ((symbolType = _), chart)
      : symbolType; // Function to update symbol type

  return chart;
};
